@@ -4,10 +4,11 @@ from qualitative_coding.corpus import QCCorpus
 from pathlib import Path
 
 @click.command(name="list")
-@click.option("-s", "--settings", default="settings.yaml", help="Settings file")
+@click.option("-s", "--settings", type=click.Path(exists=True), help="Settings file")
 def list_corpus_paths(settings):
     "List corpus paths"
-    corpus = QCCorpus(settings)
+    settings_path = settings or os.environ.get("QC_SETTINGS", "settings.yaml")
+    corpus = QCCorpus(settings_path)
     paths = []
     for dir_path, dirs, filenames in os.walk(corpus.corpus_dir):
         for fn in filenames:
