@@ -5,8 +5,10 @@
 from itertools import chain, combinations_with_replacement
 from collections import defaultdict
 from contextlib import contextmanager
+from importlib.metadata import metadata
 from pathlib import Path
 import yaml
+import shutil
 import numpy as np
 from textwrap import fill
 import os
@@ -54,7 +56,7 @@ from qualitative_coding.helpers import (
 )
 
 DEFAULT_SETTINGS = {
-    'qc_version': '1.0.3',
+    'qc_version': metadata('qualitative-coding')['version'],
     'corpus_dir': 'corpus',
     'database': 'qualitative_coding.sqlite3',
     'logs_dir': 'logs',
@@ -271,26 +273,24 @@ class QCCorpus:
     def get_code_tree_with_counts(self, 
         pattern=None,
         file_list=None,
-        coder=None,
+        coders=None,
         unit='line',
     ):
         tree = self.get_codebook()
         code_counts = self.count_codes(
             pattern=pattern,
             file_list=file_list, 
-            coder=coder,
+            coders=coders,
             unit=unit,
         )
-        for node in tree.flatten():
-            node.count = code_counts.get(node.name, 0)
 
-        def recursive_count(node):
+        def recursive_totals(node):
             for child in node.children:
-                recursive_count(child)
+                recursive_totals(child)
             node.count = code_counts.get(node.name, 0)
-            node.total = node.count + sum(c.count for c in node.children)
+            node.total = node.count + sum([c.total for c in node.children])
 
-        recursive_count(tree)
+        recursive_totals(tree)
         return tree
 
     def get_or_create_coder(self, coder_name):
@@ -303,6 +303,21 @@ class QCCorpus:
             self.get_session().commit()
             return coder
 
+    # TODO once we implement migrations, this should be simplified using a 
+    # delete cascade on the coder->coded_line
+    def delete_coder(self, coder_name):
+        try:
+            q = select(CodedLine).where(CodedLine.coder_id == coder_name)
+            coded_lines = self.get_session().scalars(q)
+            q = select(Coder).where(Coder.name == coder_name)
+            coder = self.get_session().execute(q).scalar_one()
+        except NoResultFound:
+            raise QCError(f"There is no coder named {coder_name}.")
+        for coded_line in coded_lines:
+            self.get_session().delete(coded_line)
+        self.get_session().delete(coder)
+        self.get_session().commit()
+
     def get_all_coders(self):
         q = select(Coder)
         return self.get_session().scalars(q)
@@ -328,7 +343,7 @@ class QCCorpus:
 
     def get_codebook(self):
         "Reads a tree of codes from the codebook file."
-        return TreeNode.read_yaml(self.resolve_path(self.settings['codebook']))
+        return TreeNode.read_yaml(self.codebook_path)
 
     def get_document(self, corpus_path):
         """Fetches a document object.
@@ -426,7 +441,7 @@ class QCCorpus:
         if not recursive and source.is_dir():
             raise InvalidParameter(f"{source} is a dir. Use --recursive.")
         if corpus_root and Path(corpus_root).is_absolute():
-            raise InvalidParameter(f"corpus_root ({corpus_root}) must be a relative directory.")
+            raise InvalidParameter(f"corpus_root ({corpus_root}) must be a relative path.")
 
         if corpus_root:
             dest_root_dir = self.corpus_dir / corpus_root
@@ -436,7 +451,8 @@ class QCCorpus:
 
         if recursive:
             for dir_path, dir_names, filenames in os.walk(source):
-                dest_dir = dest_root_dir / dir_path
+                rel_dir_path = str(Path(dir_path).relative_to(source))
+                dest_dir = dest_root_dir / rel_dir_path
                 dest_dir.mkdir(parents=True, exist_ok=True)
                 for fn in filenames:
                     source_path = Path(dir_path) / fn
@@ -480,7 +496,7 @@ class QCCorpus:
                 ))
         self.get_session().commit()
 
-    def count_codes(self, pattern=None, file_list=None, coder=None, unit="line"):
+    def count_codes(self, pattern=None, file_list=None, coders=None, unit="line"):
         """Returns a dict of {code:count}.
         """
         unit_column = self.get_column_to_count(unit)
@@ -490,7 +506,7 @@ class QCCorpus:
             .group_by(Code.name)
         )
         query = self.filter_query_by_document(query, pattern, file_list, unit=unit)
-        query = self.filter_query_by_coder(query, coder)
+        query = self.filter_query_by_coders(query, coders)
         result = self.get_session().execute(query).all()
         return dict(result)
 
@@ -525,17 +541,68 @@ class QCCorpus:
                     dp = Path(dir_path).relative_to(target)
                     rtarget = target / dp / fn
                     rdestination = destination / dp / fn
-                    file_path = str(rtarget.relative_to(self.corpus_dir))
-                    doc = self.get_documents(file_list=[file_path])[0]
-                    doc.file_path = str(rdestination.relative_to(self.corpus_dir))
+                    old_file_path = str(rtarget.relative_to(self.corpus_dir))
+                    new_file_path = str(rdestination.relative_to(self.corpus_dir))
+                    self._move_document(old_file_path, new_file_path)
         else:
-            file_path = str(target.relative_to(self.corpus_dir))
-            doc = self.get_documents(file_list=[file_path])[0]
-            doc.file_path = str(destination.relative_to(self.corpus_dir))
+            old_file_path = str(target.relative_to(self.corpus_dir))
+            new_file_path = str(destination.relative_to(self.corpus_dir))
+            self._move_document(old_file_path, new_file_path)
         destination.parent.mkdir(parents=True, exist_ok=True)
         target.rename(destination)
         self.get_session().commit()
 
+    def _move_document(self, old_file_path, new_file_path):
+        """Updates a document's file path, and updates DocumentIndex.document_id
+        to match. Does not commit the session.
+        """
+        doc = self.get_documents(file_list=[old_file_path])[0]
+        for document_index in doc.indices:
+            document_index.document_id = new_file_path
+        doc.file_path = new_file_path
+
+    def remove_document(self, target, recursive=False):
+        """Remove a document, or recursively remove all documents in a directory.
+        Also deletes related CodedLines.
+        """
+        session = self.get_session()
+        self.validate_corpus_paths()
+        target = self.corpus_dir / target
+        if not target.exists():
+            raise QCError(f"{target} does not exist")
+        if recursive and not target.is_dir():
+            raise QCError(f"Cannot use --recursive when {target} is a file.")
+        if not recursive and target.is_dir():
+            raise QCError(f"{target} is a directory. Use --recursive")
+        if recursive:
+            for dir_path, dir_names, filenames in os.walk(target):
+                for fn in filenames:
+                    dp = Path(dir_path).relative_to(target)
+                    rtarget = target / dp / fn
+                    file_path = str(rtarget.relative_to(self.corpus_dir))
+                    self._remove_document(file_path)
+            shutil.rmtree(target)
+        else:
+            file_path = str(target.relative_to(self.corpus_dir))
+            self._remove_document(file_path)
+            target.unlink()
+        session.commit()
+
+    def _remove_document(self, file_path):
+        """Removes document and all dependents from the database.
+        Currently, there are no delete cascades in the database. 
+        When delete cascades are added in a future migration, this 
+        method will become trivial.
+        
+        This method is intended to be called by other methods.
+        Note that it does not commit the session.
+        """
+        session = self.get_session()
+        for cl in self.get_coded_lines(file_list=[file_path]):
+            session.delete(cl)
+        doc = self.get_documents(file_list=[file_path])[0]
+        session.delete(doc)
+
     def filter_query_by_document(self, query, pattern=None, file_list=None, 
             unit="line"):
         """Filters a query by which documents match. 
@@ -554,12 +621,12 @@ class QCCorpus:
             query = query.where(Document.file_path.in_(file_list))
         return query
 
-    def filter_query_by_coder(self, query, coder=None, coded_line_alias=CodedLine):
-        """Filters a query by coder, if coder is given.
+    def filter_query_by_coders(self, query, coders=None, coded_line_alias=CodedLine):
+        """Filters a query by coders, if coders are given.
         Joins the query to CodedLine and adds a where clause matching the coder name.
         """
-        if coder:
-            return query.where(coded_line_alias.coder_id == coder)
+        if coders:
+            return query.where(coded_line_alias.coder_id.in_(coders))
         else:
             return query
 
@@ -573,11 +640,16 @@ class QCCorpus:
             "document": document_alias.file_path,
         }[unit]
 
-    def get_coded_lines(self, codes=None, pattern=None, file_list=None, coder=None):
-        """Returns (Code.name, CodedLine.line_number, Document.file_path)
+    def get_coded_lines(self, codes=None, pattern=None, file_list=None, coders=None):
+        """Returns (code, coder, line, file_path)
         """
         query = (
-            select(CodedLine.code_id, CodedLine.line, DocumentIndex.document_id)
+            select(
+                CodedLine.code_id, 
+                CodedLine.coder_id, 
+                CodedLine.line, 
+                DocumentIndex.document_id
+            )
             .join(CodedLine.locations)
             .join(Location.document_index)
             .where(DocumentIndex.name == "paragraphs")
@@ -586,15 +658,15 @@ class QCCorpus:
         if codes:
             query = query.where(CodedLine.code_id.in_(codes))
         query = self.filter_query_by_document(query, pattern, file_list)
-        query = self.filter_query_by_coder(query, coder)
+        query = self.filter_query_by_coders(query, coders)
         return self.get_session().execute(query).all()
 
-    def get_coded_paragraphs(self, codes=None, pattern=None, file_list=None, coder=None):
-        """Returns (Code.name, CodedLine.line_number, Document.file_path, Location.id,
+    def get_coded_paragraphs(self, codes=None, pattern=None, file_list=None, coders=None):
+        """Returns (Code.name, Coder.name, CodedLine.line_number, Document.file_path, Location.id,
                 Location.start_line, Location.end_line)
         """
         query = (
-            select(CodedLine.code_id, DocumentIndex.document_id,
+            select(CodedLine.code_id, CodedLine.coder_id, DocumentIndex.document_id,
                    Location.start_line, Location.end_line)
             .join(CodedLine.locations)
             .join(Location.document_index)
@@ -604,14 +676,14 @@ class QCCorpus:
         if codes:
             query = query.where(CodedLine.code_id.in_(codes))
         query = self.filter_query_by_document(query, pattern, file_list)
-        query = self.filter_query_by_coder(query, coder)
+        query = self.filter_query_by_coders(query, coders)
         return self.get_session().execute(query).all()
 
-    def get_coded_documents(self, codes=None, pattern=None, file_list=None, coder=None):
-        """Returns (Code.name, Document.file_path)
+    def get_coded_documents(self, codes=None, pattern=None, file_list=None, coders=None):
+        """Returns (Code.name, Coder.name, Document.file_path)
         """
         query = (
-            select(CodedLine.code_id, DocumentIndex.document_id)
+            select(CodedLine.code_id, CodedLine.coder_id, DocumentIndex.document_id)
             .join(CodedLine.locations)
             .join(Location.document_index)
             .where(DocumentIndex.name == "paragraphs")
@@ -620,7 +692,7 @@ class QCCorpus:
         if codes:
             query = query.where(CodedLine.code_id.in_(codes))
         query = self.filter_query_by_document(query, pattern, file_list)
-        query = self.filter_query_by_coder(query, coder)
+        query = self.filter_query_by_coders(query, coders)
         return self.get_session().execute(query).all()
 
     def get_code_matrix(self, codes, 
@@ -631,7 +703,7 @@ class QCCorpus:
         pattern=None,
         file_list=None,
         invert=False,
-        coder=None,
+        coders=None,
         expanded=False,
     ):
         """Returns a list of codes and a matrix of (codes * selections). 
@@ -670,8 +742,8 @@ class QCCorpus:
                 .join(LocationB, CodedLineB.locations)
                 .where(LocationA.document_index_id == LocationB.document_index_id)
             )
-            query = self.filter_query_by_coder(query, coder, CodedLineA)
-            query = self.filter_query_by_coder(query, coder, CodedLineB)
+            query = self.filter_query_by_coders(query, coders, CodedLineA)
+            query = self.filter_query_by_coders(query, coders, CodedLineB)
 
             ab_count = self.get_session().scalars(query).first()
             cooccurrences[ix_a][ix_b] = ab_count
@@ -692,17 +764,10 @@ class QCCorpus:
             code_tree.add_child(new_code)
         TreeNode.write_yaml(self.codebook_path, code_tree)
 
-    def rename_codes(self, old_codes, new_code, pattern=None, file_list=None, coder=None):
+    def rename_codes(self, old_codes, new_code, pattern=None, file_list=None, coders=None):
         """
         Updates the codefiles and the codebook, replacing the old code with the new code. 
         Removes the old code from the codebook.
-
-        How will I do this?
-        get all the coded lines (with relevant params).
-        For each, create a new coded line with the new code. 
-        Then delete all coded lines with the old code.
-        get_coded_lines returns: (Code.name, CodedLine.line_number, Document.file_path)
-
         """
         session = self.get_session()
         new_code = self.get_or_create_code(new_code)
@@ -714,7 +779,7 @@ class QCCorpus:
             .where(CodedLine.code_id.in_(old_codes))
         )
         query = self.filter_query_by_document(query, pattern, file_list)
-        query = self.filter_query_by_coder(query, coder)
+        query = self.filter_query_by_coders(query, coders)
         matching_coded_lines = session.execute(query).scalars()
         for cl in matching_coded_lines:
             doc_path = cl.locations[0].document_index.document_id
@@ -742,6 +807,59 @@ class QCCorpus:
         result = session.execute(query).scalars()
         return len(list(result)) > 0
 
+    def count_codes_by_coder(self, codes=None, coders=None, recursive_codes=False,
+            depth=None, pattern=None, file_list=None, unit='line', totals=True):
+        """Counts codes per-coder. Returns a dict of dicts like {"coder":{"code": n}}.
+        """
+        coders = coders or [c.name for c in self.get_all_coders()]
+        attr = "total" if totals else "count"
+        totals_by_coder = defaultdict(lambda: defaultdict(int))
+        for coder in coders:
+            tree = self.get_code_tree_with_counts(
+                pattern=pattern, 
+                file_list=file_list,
+                coders=[coder], 
+                unit=unit, 
+            )
+            if codes:
+                nodes = sum([tree.find(c) for c in codes], [])
+                if recursive_codes:
+                    nodes = set(sum([n.flatten(depth=depth) for n in nodes], []))
+            else:
+                nodes = tree.flatten(depth=depth)
+            for node in nodes:
+                totals_by_coder[coder][node.expanded_name()] = getattr(node, attr)
+        return totals_by_coder
+
+    def count_codes_by_document(self, codes=None, coders=None, recursive_codes=False,
+            depth=None, pattern=None, file_list=None, unit='line', totals=True):
+        """Counts codes per-coder. Returns a dict of dicts like {"coder":{"code": n}}.
+        """
+        documents = self.get_documents(pattern=pattern, file_list=file_list)
+        documents = [doc.file_path for doc in documents]
+        attr = "total" if totals else "count"
+        totals_by_doc = defaultdict(lambda: defaultdict(int))
+        for doc in documents:
+            tree = self.get_code_tree_with_counts(
+                file_list=[doc],
+                coders=coders, 
+                unit=unit, 
+            )
+            if codes:
+                nodes = sum([tree.find(c) for c in codes], [])
+                if recursive_codes:
+                    nodes = set(sum([n.flatten(depth=depth) for n in nodes], []))
+            else:
+                nodes = tree.flatten(depth=depth)
+            for node in nodes:
+                totals_by_doc[doc][node.expanded_name()] = getattr(node, attr)
+        return totals_by_doc
+
+
+
+
+
+
 
 
 
